import React, { useContext } from "react";
import { Card, Button, TextField } from "@mui/material";
import "@scss/login.scss";
import { Link, useNavigate } from "react-router-dom";
import { yupResolver } from "@hookform/resolvers/yup";
import * as yup from "yup";
import { useForm } from "react-hook-form";
import userContext from "@ts/userContext"
import Typography from "@theme/modules/components/Typography";
import login , { FailedLoginError } from "@api/login";

type Inputs = {
    email: string;
    password: string;
};

const schema = yup
    .object()
    .shape({
        email: yup
            .string()
            .email("Please provide a valid email")
            .required("No email was provided"),
        password: yup.string().required("No password was provided"),
    })
    .required();

export default function Login() {
    const user = useContext(userContext);
    const navigate = useNavigate();
    const {
        register,
        formState: { errors },
        handleSubmit,
    } = useForm<Inputs>({
        resolver: yupResolver(schema),
    });

    const { ref: emailRef, ...emailRegisterProps } = register("email");

    const emailProps = {
        ...emailRegisterProps,
        inputRef: emailRef,
        className: "textField",
        name: "email",
        id: "email",
        label: "Email",
        error: !!errors.email,
        helperText: errors?.email?.message,
    };

    const { ref: passRef, ...passRegisterProps } = register("password");

    const passProps = {
        ...passRegisterProps,
        inputRef: passRef,
        className: "textField",
        name: "password",
        id: "password",
        label: "Password",
        error: !!errors.password,
        helperText: errors?.password?.message,
    };

    async function onSubmit(data: Inputs) {
        console.warn("There will be an API call now with the following data");
        console.table(data);
        try {
            const token: ApiToken = await login(data.email, data.password);
            user.setToken(token);
            window.localStorage.setItem("token", token); // this could be done with a hook (right?)
            navigate("/");
        }catch(err){
            if (err instanceof FailedLoginError)
                alert(`Failed to login user.\nError code: '${err.code}', error message: '${err.message}'`)
            else
                alert('Failed to make api call');
        }
    }

    return (
        <main className="cardContainer">
            <form onSubmit={handleSubmit(onSubmit)}>
                <Card className="card">
                    <Typography className="title" variant="h4">Login</Typography>
                    <TextField {...emailProps} />
                    <TextField {...passProps} />
                    <Button type="submit" id="submit" variant="contained">
                        Login
                    </Button>
                    <Typography>
                        Forgot your <Link to="/forgot">password</Link>?
                    </Typography>
                </Card>
            </form>
        </main>
    );
}
